/*   **********************************************************************  **
 **   Copyright notice                                                       **
 **                                                                          **
 **   (c) 2003-2005 RSSOwl Development Team                                  **
 **   http://www.rssowl.org/                                                 **
 **                                                                          **
 **   All rights reserved                                                    **
 **                                                                          **
 **   This program and the accompanying materials are made available under   **
 **   the terms of the Common Public License v1.0 which accompanies this     **
 **   distribution, and is available at:                                     **
 **   http://www.rssowl.org/legal/cpl-v10.html                               **
 **                                                                          **
 **   A copy is found in the file cpl-v10.html and important notices to the  **
 **   license from the team is found in the textfile LICENSE.txt distributed **
 **   in this package.                                                       **
 **                                                                          **
 **   This copyright notice MUST APPEAR in all copies of the file!           **
 **                                                                          **
 **   Contributors:                                                          **
 **     RSSOwl - initial API and implementation (bpasero@rssowl.org)         **
 **                                                                          **
 **  **********************************************************************  */

package net.sourceforge.rssowl.util;

import net.sourceforge.rssowl.controller.dialog.SearchDialog;

import java.util.Hashtable;
import java.util.Vector;

/**
 * Class for parsing the search pattern (Operators: AND, OR, NOT).
 * 
 * @author <a href="mailto:bpasero@rssowl.org">Benjamin Pasero </a>
 * @version 1.1.3
 */
public class SearchpatternParser {

  /** Hashtable Key for the Must Key-Words */
  public static final String MUST_KEY = "AND";

  /** Hashtable Key for the Must-Not Key-Words */
  public static final String MUSTNOT_KEY = "NOT";

  /** This utility class constructor is hidden */
  private SearchpatternParser() {
  // Protect default constructor
  }

  /**
   * Generate a RegEx from the AND / OR keywords.
   * 
   * @param keys Vector holding the AND / OR Keywords
   * @return RegEx matching the keywords
   */
  public static Vector generateRegEx(Vector keys) {
    Vector regex = new Vector();
    String wrapperRegExStart = "";
    String wrapperRegExEnd = "";

    /** In case only whole word is allowed */
    if (SearchDialog.wasSearchWholeWord) {
      wrapperRegExStart = "(\\s|^)";
      wrapperRegExEnd = "(\\s|$)";
    }

    /** Generate RegEx for the words that should match the result */
    for (int a = 0; a < keys.size(); a++) {
      Vector and = (Vector) keys.get(a);

      /** Only 1 key */
      if (and.size() == 1) {
        StringBuffer sb = new StringBuffer(wrapperRegExStart);
        sb.append((String) and.get(0));
        sb.append(wrapperRegExEnd);
        regex.add(sb.toString());
      }

      /** More than 1 key. These are OR statements */
      else if (and.size() > 1) {
        StringBuffer sb = new StringBuffer(wrapperRegExStart + "(");
        sb.append((String) and.get(0));

        for (int b = 1; b < and.size(); b++) {
          sb.append("|");
          sb.append((String) and.get(b));
        }

        sb.append(")" + wrapperRegExEnd);
        regex.add(sb.toString());
      }
    }
    return regex;
  }

  /**
   * Parse the search pattern and return a Hashtable. The Hashtable contains one
   * Vector for the Keywords that should NOT match the result and one Vector
   * that should match the result (AND). The Vector for the AND-Keywords holds
   * one Vector for each OR-Keyword. For example: A pattern "Car AND Audi OR
   * Mercedes" would result in: [[Car], [Audi, Mercedes]] beeing equivalent to
   * "Car && (Audi || Mercedes)"
   * 
   * @param pattern The search pattern (e.g. "Car AND Audi NOT Ferrari")
   * @return parsed pattern holding the Keywords for AND, OR and NOT
   */
  public static Hashtable parsePattern(String pattern) {
    Hashtable parsedPattern = new Hashtable();

    /** Parse Words that should NOT match the result */
    int notIndex = pattern.indexOf(" NOT ");

    Vector not = new Vector();
    while (notIndex > 0) {
      int start = notIndex;
      int end = getClosestKeyword(pattern, notIndex);
      String notPattern = pattern.substring(start, end);
      String notWords[] = notPattern.split(" NOT ");

      /** Remove the NOT statement from the pattern */
      pattern = pattern.replaceFirst(notPattern, "");

      /** Add NOT Keywords to Vector */
      for (int a = 1; a < notWords.length; a++) {
        if (!not.contains(notWords[a]))
          not.add(notWords[a]);
      }
      notIndex = pattern.indexOf(" NOT ", notIndex + 1);
    }

    Vector and = parseMustAndMayWords(pattern);
    parsedPattern.put(MUST_KEY, and);
    parsedPattern.put(MUSTNOT_KEY, not);
    return parsedPattern;
  }

  /**
   * Finds the next index of one of the keywords AND, OR
   * 
   * @param pattern The search pattern
   * @param notIndex Current position in search pattern
   * @return Closest index to next keyword
   */
  private static int getClosestKeyword(String pattern, int notIndex) {
    int end = -1;
    int nextAndIndex = pattern.indexOf(" AND ", notIndex);
    int nextOrIndex = pattern.indexOf(" OR ", notIndex);

    /** Next keyword is OR */
    if ((nextOrIndex < nextAndIndex && nextOrIndex > 0) || nextAndIndex < 0)
      end = nextOrIndex;

    /** Next keyword is AND */
    else if ((nextAndIndex < nextOrIndex && nextAndIndex > 0) || nextOrIndex < 0)
      end = nextAndIndex;

    if (end < 1)
      end = pattern.length();
    return end;
  }

  /**
   * Parse Words that should match the result (AND / OR)
   * 
   * @param pattern The search pattern (NOT words are removed)
   * @return Vector holding AND plus OR keywords
   */
  private static Vector parseMustAndMayWords(String pattern) {
    String[] andPattern = pattern.split(" AND ");
    Vector and = new Vector();
    Vector or;

    for (int a = 0; a < andPattern.length; a++) {
      or = new Vector();
      String[] orPattern = andPattern[a].split(" OR ");
      for (int b = 0; b < orPattern.length; b++) {
        or.add(orPattern[b]);
      }
      and.add(or);
    }
    return and;
  }
}