/*   **********************************************************************  **
 **   Copyright notice                                                       **
 **                                                                          **
 **   (c) 2003-2005 RSSOwl Development Team                                  **
 **   http://www.rssowl.org/                                                 **
 **                                                                          **
 **   All rights reserved                                                    **
 **                                                                          **
 **   This program and the accompanying materials are made available under   **
 **   the terms of the Common Public License v1.0 which accompanies this     **
 **   distribution, and is available at:                                     **
 **   http://www.rssowl.org/legal/cpl-v10.html                               **
 **                                                                          **
 **   A copy is found in the file cpl-v10.html and important notices to the  **
 **   license from the team is found in the textfile LICENSE.txt distributed **
 **   in this package.                                                       **
 **                                                                          **
 **   This copyright notice MUST APPEAR in all copies of the file!           **
 **                                                                          **
 **   Contributors:                                                          **
 **     RSSOwl - initial API and implementation (bpasero@rssowl.org)         **
 **                                                                          **
 **  **********************************************************************  */

package net.sourceforge.rssowl.util.document;

import net.sourceforge.rssowl.controller.GUI;
import net.sourceforge.rssowl.model.Channel;
import net.sourceforge.rssowl.model.Enclosure;
import net.sourceforge.rssowl.model.NewsItem;
import net.sourceforge.rssowl.util.DateParser;
import net.sourceforge.rssowl.util.shop.StringShop;

import org.jdom.Text;

import com.lowagie.text.Anchor;
import com.lowagie.text.Cell;
import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Element;
import com.lowagie.text.HeaderFooter;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;
import com.lowagie.text.Table;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfDestination;
import com.lowagie.text.pdf.PdfOutline;
import com.lowagie.text.pdf.PdfPageEventHelper;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.rtf.RtfWriter;

import java.awt.Color;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Hashtable;

/**
 * Class to generate PDF / RTF / HTML from a Newsfeed
 * 
 * @author <a href="mailto:bpasero@rssowl.org">Benjamin Pasero </a>
 * @version 1.1.3
 */
public class DocumentGenerator {

  /** Constant to indicate a HTML formatted document */
  public static final int HTML_FORMAT = 1;

  /** Constant to indicate a PDF formatted document */
  public static final int PDF_FORMAT = 0;

  /** Constant to indicate a RTF formatted document */
  public static final int RTF_FORMAT = 2;

  /** Value to decide wether a Table fits a page or not */
  private static final int MAX_CHARS_PER_PAGE = 3000;

  /** Tokeniter to split Feed Title and News Title */
  private static final String TOKENIZER = " =;;= ";

  /** Document to write */
  private Document document;

  /** The config for the document containing Fonts */
  private DocumentConfig documentConfig;

  /** Chosen format (PDF, RTF or HTML) */
  private int format;

  /** The Channel to generate the PDF / RTF / HTML from */
  private Channel rssChannel;

  /** Store Chapters for each newsfeed that is generated to document */
  Hashtable chapters;

  /**
   * Instantiate a new DocumentGenerator
   * 
   * @param rssChannel The selected Channel
   * @param fileOS File to write document into
   * @param format PDF, RTF or HTML
   * @throws DocumentException If an error occurs
   */
  public DocumentGenerator(Channel rssChannel, FileOutputStream fileOS, int format) throws DocumentException {
    this.rssChannel = rssChannel;
    this.format = format;
    chapters = new Hashtable();
    document = new Document();
    documentConfig = DocumentConfig.getInstance();
    initDocument(fileOS);
  }

  /**
   * Get a suitable file format name for the given format.
   * 
   * @param format Any of the allowed formats PDF, RTF or HTML
   * @return String The format as file name
   */
  public static String formatToString(int format) {
    if (format == PDF_FORMAT)
      return "pdf";
    else if (format == RTF_FORMAT)
      return "rtf";
    else if (format == HTML_FORMAT)
      return "html";
    return "pdf";
  }

  /**
   * Write header and a single news to the document
   * 
   * @param newsItemOrder The sort order of the selected newsfeed
   * @throws DocumentException If an warning occurs
   */
  public void createDocument(ArrayList newsItemOrder) throws DocumentException {
    DocumentException documentException = null;

    /** Meta Infos */
    writeMetaInfos();

    try {

      /** Footer */
      writeFooter();

      /** Open document to write */
      document.open();

      /** Write the news */
      writeNews(newsItemOrder);
    }

    /** An error has occured */
    catch (DocumentException e) {
      documentException = e;
    }

    /** Close the document */
    document.close();

    /** Throw the exception if one occured */
    if (documentException != null)
      throw documentException;
  }

  /**
   * Init the document to PDF, HTML or RTF and with the given file
   * 
   * @param file The path to the file where to save the document after
   * generation
   * @throws DocumentException If an error occurs
   */
  private void initDocument(FileOutputStream file) throws DocumentException {

    /** Create new PDF document */
    if (format == PDF_FORMAT) {

      /** In order to create an Outline, we listen for Generic Tags */
      PdfWriter.getInstance(document, file).setPageEvent(new PdfPageEventHelper() {
        public void onGenericTag(PdfWriter writer, Document document, Rectangle rect, String text) {

          /** Get Root Outline Object */
          PdfContentByte cb = writer.getDirectContent();
          PdfOutline root = cb.getRootOutline();

          /** Get Feed Title and News Title from generic Tag */
          String feedTitle = text.split(TOKENIZER)[0];
          String newsTitle = text.split(TOKENIZER)[1];

          /**
           * A new Feed was reached, create Chapter for it and let it be opened.
           * Place News Title as Sub-Chapter to that Chapter.
           */
          if (!chapters.containsKey(feedTitle)) {
            PdfOutline chapter = new PdfOutline(root, new PdfDestination(PdfDestination.FIT, rect.top(-20)), feedTitle, true);
            chapters.put(feedTitle, chapter);
            new PdfOutline(chapter, new PdfDestination(PdfDestination.FIT, rect.top(-20)), newsTitle);
          }

          /**
           * News Item from Feed that already has a chapter. Add the news to the
           * Chapter of the Feed
           */
          else {
            new PdfOutline((PdfOutline) chapters.get(feedTitle), new PdfDestination(PdfDestination.FIT, rect.top(-20)), newsTitle);
          }
        }
      });

      /** Create a new HTML document */
    } else if (format == HTML_FORMAT) {

      /** Do not explicitly set the BASE Tag, since this is an aggregation */
      if (rssChannel.isAggregatedCat() || !StringShop.isset(rssChannel.getLink()))
        ExtendedHtmlWriter.getInstance(document, file);

      /** Explicitly set the BASE Tag to resolve relative Links */
      else
        ((ExtendedHtmlWriter) ExtendedHtmlWriter.getInstance(document, file)).setBase(rssChannel.getLink());
    }

    /** Create a new RTF document */
    else if (format == RTF_FORMAT)
      RtfWriter.getInstance(document, file);
  }

  /**
   * Write footer into document
   */
  private void writeFooter() {
    Chunk footerChunk = new Chunk(DateParser.formatDate() + " - " + GUI.i18n.getTranslation("DOCUMENT_GENERATED_FROM") + " (http://www.rssowl.org)", documentConfig.getItalicFont());
    HeaderFooter footer = new HeaderFooter(new Phrase(footerChunk), false);
    footer.setAlignment(Element.ALIGN_CENTER);
    footer.setBorder(1);
    document.setFooter(footer);
  }

  /** Write some meta informations to the document */
  private void writeMetaInfos() {

    /** Title */
    if (rssChannel.getTitle() != null && !rssChannel.getTitle().equals(""))
      document.addTitle(rssChannel.getTitle());

    /** Publisher */
    if (rssChannel.getManagingEditor() != null && !rssChannel.getManagingEditor().equals(""))
      document.addAuthor(rssChannel.getManagingEditor());
    else if (rssChannel.getPublisher() != null && !rssChannel.getPublisher().equals(""))
      document.addAuthor(rssChannel.getPublisher());
    else if (rssChannel.getCreator() != null && !rssChannel.getCreator().equals(""))
      document.addAuthor(rssChannel.getCreator());

    /** Date */
    document.addCreationDate();

    /** Creator */
    document.addCreator(GUI.i18n.getTranslation("DOCUMENT_GENERATED_FROM") + " (http://www.rssowl.org)");
  }

  /**
   * Write all news to the document
   * 
   * @param newsItemsOrder The sort order of the newsitems
   * @throws DocumentException If an error occurs
   */
  private void writeNews(ArrayList newsItemsOrder) throws DocumentException {

    /** Channel News Items */
    Hashtable newsItems = rssChannel.getItems();

    /** Write all newsitems in their sort order into document */
    for (int a = 0; a < newsItemsOrder.size(); a++) {
      NewsItem rssNewsItem = (NewsItem) newsItems.get(newsItemsOrder.get(a));
      String newsTitle = GUI.i18n.getTranslation("NO_TITLE");

      if (StringShop.isset(rssNewsItem.getTitle()))
        newsTitle = rssNewsItem.getTitle();

      /** Empty Spacer between News Tables */
      if (format == HTML_FORMAT)
        document.add(new Paragraph("\n"));
      else
        document.add(new Paragraph());

      /** Write this newsitem */
      writeNews(rssNewsItem, newsTitle);
    }
  }

  /**
   * Write a newsitem to the document
   * 
   * @param rssNewsItem NewsItem to write
   * @param newsTitle Title of the newsitem
   * @throws DocumentException if an error occurs
   */
  private void writeNews(NewsItem rssNewsItem, String newsTitle) throws DocumentException {

    /** Create the Table to hold the news */
    Table table = new Table(1, 2);
    table.setBorder(0);
    table.setPadding((format == HTML_FORMAT) ? 12 : 7);
    table.setWidth((format == RTF_FORMAT) ? 80 : 100);

    /** Cell for the News Title and Feed Information */
    Cell titleCell = new Cell();
    titleCell.setBorderColor(Color.LIGHT_GRAY);

    /** Feed Title */
    Paragraph title = new Paragraph();

    /** If Format is PDF, use Chunks */
    if (format == PDF_FORMAT) {
      Chunk titleChunk = new Chunk(newsTitle);

      /** Assign Newslink to Title if available */
      if (rssNewsItem.getLink() != null && !rssNewsItem.getLink().equals("")) {
        titleChunk.setFont(documentConfig.getLinkFont());
        titleChunk.setAnchor(rssNewsItem.getLink());
      }

      /** News Link not given */
      else {
        titleChunk.setFont(documentConfig.getBoldFont());
      }

      /** Set generic Tag to write Chapters and Sections */
      String feedTitle = (rssNewsItem.getNewsfeedTitle() != null && !rssNewsItem.getNewsfeedTitle().equals("")) ? rssNewsItem.getNewsfeedTitle() : GUI.i18n.getTranslation("NO_TITLE");
      titleChunk.setGenericTag(feedTitle + TOKENIZER + newsTitle);
      title.add(titleChunk);
    }

    /** If Format is not PDF, use Anchors */
    else {

      /** Assign Newslink to Title if available */
      if (rssNewsItem.getLink() != null && !rssNewsItem.getLink().equals("")) {
        Anchor titleChunk = new Anchor(newsTitle, documentConfig.getLinkFont());
        titleChunk.setReference(rssNewsItem.getLink());
        title.add(titleChunk);
      }

      /** News Link not given */
      else {
        Anchor titleChunk = new Anchor(newsTitle, documentConfig.getBoldFont());
        title.add(titleChunk);
      }
    }

    /** Sub Title containing additional information about the News */
    StringBuffer subTitleContent = new StringBuffer();

    /** Publish Date */
    if (rssNewsItem.getPubDateParsed() != null)
      subTitleContent.append(DateParser.formatDate(rssNewsItem.getPubDateParsed(), true) + " - ");
    else if (StringShop.isset(rssNewsItem.getPubDate()))
      subTitleContent.append(rssNewsItem.getPubDate() + " - ");

    /** Newsfeed Title */
    if (rssNewsItem.getNewsfeedTitle() != null && !rssNewsItem.getNewsfeedTitle().equals(""))
      subTitleContent.append(rssNewsItem.getNewsfeedTitle());

    /** Sub Title container */
    Paragraph subTitle = new Paragraph();
    Anchor subTitleChunk = new Anchor(subTitleContent.toString(), documentConfig.getSubTitleFont());

    /** Assign Feed Link to Sub Title if available */
    if (StringShop.isset(rssChannel.getHomepage()))
      subTitleChunk.setReference(rssChannel.getHomepage());

    subTitle.add(subTitleChunk);

    /** Add Title and Sub Title to Title Cell */
    titleCell.add(title);

    /** Show Sub-Title beyond Title if PDF Format */
    if (format == PDF_FORMAT)
      titleCell.add(subTitle);

    /** Add Title Cell to Table */
    table.addCell(titleCell);

    /** Cell containing News description */
    Cell descriptionCell = new Cell();
    descriptionCell.setBorderColor(Color.LIGHT_GRAY);

    /** NewsItem content containing description, comments, source... */
    StringBuffer description = new StringBuffer();

    /** Description */
    if (StringShop.isset(rssNewsItem.getDescription()))
      description.append(Text.normalizeString(rssNewsItem.getDescription()));

    /** Enclosures of the news */
    if (rssNewsItem.getEnclosures().size() > 0) {
      description.append("\n\n" + GUI.i18n.getTranslation("NEWS_ITEM_INFO_ENCLOSURE") + ": ");

      /** Write all enclosures to document */
      for (int c = 0; c < rssNewsItem.getEnclosures().size(); c++) {
        Enclosure enclosure = (Enclosure) rssNewsItem.getEnclosures().get(c);
        description.append("\n" + enclosure.getUrl() + " (" + enclosure.getType() + " / " + enclosure.getLength(true) + ")");
      }
    }

    /** Source of the news */
    if (rssNewsItem.getSource() != null && !rssNewsItem.getSource().equals(""))
      description.append("\n\n" + GUI.i18n.getTranslation("NEWS_ITEM_INFO_SOURCE") + ": " + rssNewsItem.getSource());

    /** Comments of the news */
    if (rssNewsItem.getComments().size() > 0) {
      description.append("\n\n" + GUI.i18n.getTranslation("NEWS_ITEM_INFO_COMMENTS") + ":\n");

      /** Write all comments to the document */
      for (int b = 0; b < rssNewsItem.getComments().size(); b++)
        if (rssNewsItem.getComments().get(b) != null && !rssNewsItem.getComments().get(b).equals(""))
          description.append("\n" + rssNewsItem.getComments().get(b));
    }

    /** In case description is empty */
    if (description.length() == 0)
      description.append(GUI.i18n.getTranslation("NEWS_NO_DESCRIPTION"));

    /** Description Container. On HTML explicitly set the Font */
    Paragraph descriptionContainer = (format != HTML_FORMAT) ? new Paragraph() : new Paragraph("", documentConfig.getNormalFont());

    /** In case the Format is not HTML, strip all Tags from the Text */
    Chunk descriptionChunk;
    if (format == HTML_FORMAT)
      descriptionChunk = new Chunk(description.toString(), documentConfig.getNormalFont());
    else
      descriptionChunk = new Chunk(StringShop.stripTags(description.toString()), documentConfig.getNormalFont());

    /** Add Chunk to Container */
    descriptionContainer.add(descriptionChunk);

    /** Add Description to Description Cell */
    descriptionCell.add(descriptionContainer);

    /** Add Author to Cell if available */
    if (rssNewsItem.getAuthor() != null && !rssNewsItem.getAuthor().equals("")) {
      String author = rssNewsItem.getAuthor().replaceAll("mailto:", "");
      Paragraph authorContainer = (format != HTML_FORMAT) ? new Paragraph() : new Paragraph("", documentConfig.getSmallItalicFont());
      Chunk authorChunk = new Chunk((format == PDF_FORMAT) ? author : "\n" + author, documentConfig.getSmallItalicFont());
      authorContainer.add(authorChunk);

      /** Add Author to Description Cell */
      descriptionCell.add(authorContainer);
    }

    /** If Format is HTML or RTF, add Sub-Title bottom-right to Description */
    if (format != PDF_FORMAT) {
      subTitle.setAlignment(Element.ALIGN_RIGHT);
      descriptionCell.add(subTitle);
    }

    /** Add Description Cell to Table */
    table.addCell(descriptionCell);

    /** Decide wether the table should fit the page */
    table.setTableFitsPage(descriptionChunk.content().length() < MAX_CHARS_PER_PAGE && format != RTF_FORMAT);

    /** Add Table to Document */
    document.add(table);
  }
}