/*   **********************************************************************  **
 **   Copyright notice                                                       **
 **                                                                          **
 **   (c) 2003-2005 RSSOwl Development Team                                  **
 **   http://www.rssowl.org/                                                 **
 **                                                                          **
 **   All rights reserved                                                    **
 **                                                                          **
 **   This program and the accompanying materials are made available under   **
 **   the terms of the Common Public License v1.0 which accompanies this     **
 **   distribution, and is available at:                                     **
 **   http://www.rssowl.org/legal/cpl-v10.html                               **
 **                                                                          **
 **   A copy is found in the file cpl-v10.html and important notices to the  **
 **   license from the team is found in the textfile LICENSE.txt distributed **
 **   in this package.                                                       **
 **                                                                          **
 **   This copyright notice MUST APPEAR in all copies of the file!           **
 **                                                                          **
 **   Contributors:                                                          **
 **     RSSOwl - initial API and implementation (bpasero@rssowl.org)         **
 **                                                                          **
 **  **********************************************************************  */

package net.sourceforge.rssowl.util.document;

import net.sourceforge.rssowl.util.shop.StringShop;

import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.Element;
import com.lowagie.text.Image;
import com.lowagie.text.html.HtmlTags;
import com.lowagie.text.html.HtmlWriter;
import com.lowagie.text.markup.MarkupTags;

import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Properties;

/**
 * This class extends the HtmlWriter of iText and will prevent the write()
 * Method to encode the written String and escape HTML tags.
 * 
 * @author <a href="mailto:bpasero@rssowl.org">Benjamin Pasero </a>
 * @version 1.1.3
 */
public class ExtendedHtmlWriter extends HtmlWriter {

  /**
   * Constructs a HtmlWriter.
   * 
   * @param doc The Document that has to be written as HTML
   * @param os The OutputStream the writer has to write to.
   */
  protected ExtendedHtmlWriter(Document doc, OutputStream os) {
    super(doc, os);
  }

  /**
   * Gets an instance of the ExtendedHtmlWriter
   * 
   * @param document The Document that has to be written
   * @param os The OutputStream the writer has to write to.
   * @return a new HtmlWriter
   */

  public static HtmlWriter getInstance(Document document, OutputStream os) {
    return new ExtendedHtmlWriter(document, os);
  }

  /**
   * Writes a base reference in the header.
   * 
   * @param url The URL of the BASE reference
   */

  public void setBase(String url) {
    try {
      addTabs(2);
      writeStart("base");
      write("href", url);
      writeEnd();
    } catch (IOException e) {
      return;
    }
  }

  /**
   * Since the ExtendedHtmlWriter is not calling the HtmlEncoder.encode() Method
   * we have to offer our own implementation. This is required, because newlines
   * have to be replaced with the HTML break tag BR.
   * 
   * @param str The String to encode
   * @return String the Encoded String
   */
  private String simpleHtmlEncode(String str) {
    str = StringShop.replaceAll(str, "\n", "<br />");
    return str;
  }

  /**
   * Writes the HTML representation of an element. The case "Chunk" is extended
   * and the call to HtmlEncoder.encode() was removed in order to avoid having
   * HTML escaped.
   * 
   * @param element the element
   * @param indent the indentation
   */

  protected void write(Element element, int indent) throws IOException {
    Properties styleAttributes = null;
    switch (element.type()) {

      /** Extended Treatment for Element.CHUNK */
      case Element.CHUNK: {
        Chunk chunk = (Chunk) element;

        /** if the chunk contains an image, return the image representation */
        Image image = chunk.getImage();
        if (image != null) {
          write(image, indent);
          return;
        }

        if (chunk.isEmpty())
          return;
        HashMap attributes = chunk.getAttributes();
        if (attributes != null && attributes.get(Chunk.NEWPAGE) != null) {
          return;
        }

        boolean tag = isOtherFont(chunk.font()) || hasMarkupAttributes(chunk) || styleAttributes != null;
        if (tag) {
          addTabs(indent);
          writeStart(MarkupTags.SPAN);
          if (isOtherFont(chunk.font())) {
            write(chunk.font(), styleAttributes);
          }
          if (hasMarkupAttributes(chunk)) {
            writeMarkupAttributes(chunk);
          }
          os.write(GT);
        }
        if (attributes != null && attributes.get(Chunk.SUBSUPSCRIPT) != null) {
          if (((Float) attributes.get(Chunk.SUBSUPSCRIPT)).floatValue() > 0) {
            writeStart(HtmlTags.SUP);
          } else {
            writeStart(HtmlTags.SUB);
          }
          os.write(GT);
        }

        /** Extended call, was write(HtmlEncoder.encode(chunk.content())) */
        write(simpleHtmlEncode(chunk.content()));

        if (attributes != null && attributes.get(Chunk.SUBSUPSCRIPT) != null) {
          os.write(LT);
          os.write(FORWARD);
          if (((Float) attributes.get(Chunk.SUBSUPSCRIPT)).floatValue() > 0) {
            write(HtmlTags.SUP);
          } else {
            write(HtmlTags.SUB);
          }
          os.write(GT);
        }
        if (tag) {
          writeEnd(MarkupTags.SPAN);
        }
        return;
      }

      /** In any other case, use super implementation */
      default:
        super.write(element, indent);
    }
  }
}